﻿' **** Minesweeper BASIC
' *** Written by Lukman Hakim
' *** March, 2009

Public Class minesweeperBoard

    Public Enum level As Integer
        EASY
        MEDIUM
        EXPERT
    End Enum

    Public Enum gameStatus As Integer
        LOSE
        WIN
        NOT_DECIDED
    End Enum

    Public mineNumber As Integer
    Public numOfMinePoint As Integer
    Public maxCol As Integer, maxRow As Integer
    Public boardSize As Size

    Public minePoint() As minesweeperPoint

    Private Function getSurroundMineNumber(ByVal index As Integer) As Integer
        Dim xOffset As Integer, yOffset As Integer
        Dim trialCol As Integer, trialRow As Integer, trialIndexNum As Integer
        Dim mineNumber As Integer = 0
        Dim col As Integer, row As Integer

        col = (index Mod maxCol)
        row = Math.Floor(index / maxCol)

        For yOffset = -1 To 1

            For xOffset = -1 To 1

                If Not ((xOffset = 0) And (yOffset = 0)) Then

                    trialCol = col + xOffset
                    trialRow = row + yOffset

                    ' **** boundary condition
                    If ((trialCol >= 0) And (trialCol <= maxCol - 1)) And _
                        ((trialRow >= 0) And (trialRow <= maxRow - 1)) Then

                        trialIndexNum = trialRow * maxCol + trialCol

                        If (minePoint(trialIndexNum).isBomb) Then mineNumber = mineNumber + 1

                    End If

                End If

            Next xOffset

        Next yOffset

        Return mineNumber

    End Function

    Public Sub mapMineNumber()
        Dim i As Integer

        For i = 0 To numOfMinePoint - 1
            minePoint(i).mineAround = getSurroundMineNumber(i)
        Next

    End Sub

    Public Function getBombNumberAssumption() As Integer
        Dim bombNumAssumption As Integer = mineNumber

        For i = 0 To numOfMinePoint - 1

            If minePoint(i).flagStatus = minesweeperPoint.flag.FLAGGED Then

                bombNumAssumption = bombNumAssumption - 1

            End If

        Next

        Return bombNumAssumption

    End Function

    Private Sub showBomb(ByVal showMistake As Boolean)
        Dim i As Integer

        For i = 0 To numOfMinePoint - 1

            If (minePoint(i).isBomb) Then

                minePoint(i).opened = True

            ElseIf (showMistake) Then

                If (minePoint(i).flagStatus = minesweeperPoint.flag.FLAGGED) Then
                    minePoint(i).opened = True
                End If

            End If

        Next

    End Sub

    Private Sub processBlankPoint()
        Dim i As Integer
        Dim xOffset As Integer, yOffset As Integer
        Dim col As Integer, row As Integer
        Dim trialCol As Integer, trialRow As Integer, trialIndex As Integer
        Dim allBlankPointChecked As Boolean

        Do

            allBlankPointChecked = True

            For i = 0 To numOfMinePoint - 1

                ' *** if i is opened and it is blank
                If (minePoint(i).opened) And (minePoint(i).mineAround = 0) Then

                    col = (i Mod maxCol)
                    row = Math.Floor(i / maxCol)

                    For yOffset = -1 To 1
                        For xOffset = -1 To 1

                            If Not (xOffset = 0 And yOffset = 0) Then

                                trialCol = col + xOffset
                                trialRow = row + yOffset

                                If ((trialCol >= 0) And (trialCol <= maxCol - 1)) And _
                                    ((trialRow >= 0) And (trialRow <= maxRow - 1)) Then

                                    trialIndex = trialRow * maxCol + trialCol

                                    If (Not (minePoint(trialIndex).opened)) And (Not (minePoint(trialIndex).isBomb)) Then

                                        minePoint(trialIndex).opened = True
                                        minePoint(trialIndex).flagStatus = minesweeperPoint.flag.NONE

                                        If (minePoint(trialIndex).mineAround = 0) Then allBlankPointChecked = False

                                    End If

                                End If

                            End If

                        Next xOffset
                    Next yOffset

                End If

            Next

        Loop Until allBlankPointChecked

    End Sub

    Public Function openMinePoint(ByVal pointIndex As Integer) As Integer

        With minePoint(pointIndex)

            .opened = True

            If .isBomb Then

                .flagStatus = minesweeperPoint.flag.HITTED
                Call showBomb(True)
                Return gameStatus.LOSE

            Else

                .flagStatus = minesweeperPoint.flag.NONE

                If (.mineAround = 0) Then

                    Call processBlankPoint()

                End If

                If (checkWin()) Then

                    Return gameStatus.WIN

                End If

            End If

        End With

        Return gameStatus.NOT_DECIDED

    End Function


    Private Sub setBoardProperties(ByVal reqLevel As Integer)

        Select Case reqLevel

            Case level.EASY
                maxCol = 10
                maxRow = 10
                mineNumber = 10

            Case level.MEDIUM
                maxCol = 15
                maxRow = 15
                mineNumber = 40

            Case level.EXPERT
                maxCol = 18
                maxRow = 18
                mineNumber = 80

        End Select

        boardSize = New Size(maxCol * minesweeperPoint.MINEPOINT_WIDTH, maxRow * minesweeperPoint.MINEPOINT_HEIGHT)
        numOfMinePoint = maxCol * maxRow

    End Sub

    Private Sub generateMinePoint()

        Dim i As Integer
        Dim reqMineNum As Integer, trialPoint As Integer

        ' **** prepare the mine point object
        reqMineNum = mineNumber
        ReDim minePoint(numOfMinePoint)

        For i = 0 To numOfMinePoint - 1
            minePoint(i) = New minesweeperPoint
        Next

        ' *** put mine randomly on mine point
        Do

            Randomize()
            trialPoint = Int(Rnd() * numOfMinePoint)
            If Not (minePoint(trialPoint).isBomb) Then
                minePoint(trialPoint).isBomb = True
                reqMineNum = reqMineNum - 1
            End If

        Loop Until (reqMineNum <= 0)

    End Sub

    Public Sub New(ByVal reqLevel As Integer)

        Call setBoardProperties(reqLevel)
        Call generateMinePoint() ' *** set mines
        Call mapMineNumber() ' *** map number of mines

    End Sub

    Public Function checkWin() As Boolean

        Dim i As Integer, countClosed As Integer = 0

        For i = 0 To numOfMinePoint - 1
            If Not (minePoint(i).opened) Then
                countClosed = countClosed + 1
            End If
        Next

        If (countClosed = mineNumber) Then
            Return True
            Call showBomb(False)
        Else
            Return False
        End If

    End Function

End Class





Public Class minesweeperPoint

    Enum flag As Integer
        NONE
        FLAGGED
        QUESTIONED
        HITTED
    End Enum

    Public flagStatus As Integer
    Public mineAround As Integer
    Public isBomb As Boolean
    Public opened As Boolean

    Public Const MINEPOINT_WIDTH As Integer = 25
    Public Const MINEPOINT_HEIGHT As Integer = 25

    Public Shared LETTER_COLOR() As Color = {Color.Black, Color.LightSkyBlue, Color.Lime, Color.Red, Color.Cyan, _
                                                Color.Aqua, Color.LightPink, Color.Orange, Color.White}

    Public Sub New()
        isBomb = False
        flagStatus = flag.NONE
        mineAround = 0
        opened = False
    End Sub

    Public Shared Function drawTile() As Bitmap
        Dim lgBrush As System.Drawing.Drawing2D.LinearGradientBrush

        Dim g As Graphics, b As Bitmap
        Dim optRect As New RectangleF(0, 0, minesweeperPoint.MINEPOINT_WIDTH, minesweeperPoint.MINEPOINT_HEIGHT)

        b = New Bitmap(minesweeperPoint.MINEPOINT_WIDTH, minesweeperPoint.MINEPOINT_HEIGHT)

        lgBrush = New Drawing2D.LinearGradientBrush(New Point(minesweeperPoint.MINEPOINT_WIDTH / 2, minesweeperPoint.MINEPOINT_HEIGHT / 3), _
                                                   New Point(minesweeperPoint.MINEPOINT_WIDTH / 2, minesweeperPoint.MINEPOINT_HEIGHT), _
                                                   Color.White, Color.Gray)

        g = Graphics.FromImage(b)
        g.FillRectangle(lgBrush, optRect)
        g.FillRectangle(New SolidBrush(Color.WhiteSmoke), optRect.X, optRect.Y, optRect.Width, optRect.Height / 3)
        g.DrawRectangle(New Pen(Color.Black, 2), 2, 2, minesweeperPoint.MINEPOINT_WIDTH - 4, minesweeperPoint.MINEPOINT_HEIGHT - 4)
        g.DrawRectangle(New Pen(Color.FromArgb(200, 255, 255, 255), 1), 3, 3, minesweeperPoint.MINEPOINT_WIDTH - 6, minesweeperPoint.MINEPOINT_HEIGHT - 6)
        g.Dispose()
        Return b

    End Function

    Public Shared Function drawBomb() As Bitmap
        Const padding = 5
        Dim g As Graphics, b As Bitmap
        Dim optRect As New RectangleF(padding, padding, _
                                      minesweeperPoint.MINEPOINT_WIDTH - (2 * padding), _
                                      minesweeperPoint.MINEPOINT_HEIGHT - (2 * padding))


        b = New Bitmap(minesweeperPoint.MINEPOINT_WIDTH, minesweeperPoint.MINEPOINT_HEIGHT)

        g = Graphics.FromImage(b)
        g.FillEllipse(New SolidBrush(Color.FromArgb(255, 255, 0, 0)), optRect)

        g.DrawEllipse(New Pen(Color.FromArgb(255, 128, 0, 0), 2), optRect)

        g.Dispose()
        Return b

    End Function

End Class
